from unittest import TestCase


class AuthHelperTestCase(TestCase):
    """
    Implement some custom tests functions
    """

    def assertAnyIn(self, needles, haystack):
        """
        Asserts wether any of needles exists in haystack
        :param list needles: Collection what we are searching for
        :param list haystack: Source of truth
        :return:
        """
        if not any(x in haystack for x in needles):
            raise AssertionError('None of \'{needles}\' in \'{haystack}\''.format(
                    needles=",".join(needles),
                    haystack=haystack
            ))



from random import randrange, choice, shuffle, random, sample
from math import floor, ceil

__author__ = 'drews'

SYMBOLS = "!@#$%^&*()_+-=[]{}|'"
LOWERCASE = "abcdefghijklmnopqrstuvwxyz"
UPPERCASE = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
NUMBERS = "0123456789"


def generate(password_policy):
    """
    Builds a password based on the password policy provided
    :param password_policy iam.AccountPasswordPolicy():
    :return basestring:
    """

    # Generate a password between: the specified min length; and 130% of the min length with a minimum
    # of 2 chars added.
    minimum_password_length = password_policy.minimum_password_length
    password_length = randrange(
        minimum_password_length,
        max(int(minimum_password_length * 1.3), minimum_password_length + 2)
    )

    # Extract the policy requirements
    policy_requirements = {
        'require_symbols': password_policy.require_symbols,
        'require_lowercase_characters': password_policy.require_lowercase_characters,
        'require_uppercase_characters': password_policy.require_uppercase_characters,
        'require_numbers': password_policy.require_numbers
    }

    # If the requirement was False, it means it does not *need* to be enforced but, that it *could* be enforced.
    for policy_key, setting in policy_requirements.items():
        if not policy_requirements[policy_key]:
            policy_requirements[policy_key] = choice([True, False])

    # If we have no policy requirements, make sure to set at least 2 policy requirements, with a maximum of all
    if sum(policy_requirements.values()) < 2:
        num_policies_to_set = randrange(2, len(policy_requirements) + 1)
        policies_to_set = sample(policy_requirements.keys(), num_policies_to_set)
        for policy_to_set in policies_to_set:
            policy_requirements[policy_to_set] = True

    # Number of policies to enforce
    number_of_char_groups = sum(policy_requirements.values())

    # Create a list of policies to enforce, and specify how many characters in the password will be composed of each
    # policy.
    char_num_groups = []
    for i in range(1, number_of_char_groups):
        char_num_groups.append(int(ceil(random() * (2 * float(password_length)) / float(number_of_char_groups))))
    char_num_groups.append(password_length - sum(char_num_groups))

    # There will be times where the above algorithm, will create a negative value as the last entry.
    # For these cases, we remove the negative and split the largest value in the array.
    if len([n for n in char_num_groups if n > 0]) != len(char_num_groups):
        # Filter the negative value
        char_num_groups = [n for n in char_num_groups if n > 0]
        # Get the biggest value
        biggest_group_num = max(char_num_groups)
        char_num_groups.remove(biggest_group_num)
        # Split it and re-append to list
        new_group1 = int(biggest_group_num / 2)
        new_group2 = biggest_group_num - new_group1
        char_num_groups.append(new_group1)
        char_num_groups.append(new_group2)

    def choose_n_chars(n, char):
        """
        Choose n characters at random from char, and return them
        :param n int: number of characters to return
        :param char list[str]: pool of characters to choose from
        :return:
        """
        return [choice(char) for x in range(n)]

    password_composition = []
    shuffle(char_num_groups)
    if policy_requirements['require_symbols']:
        password_composition.extend(choose_n_chars(char_num_groups.pop(0), SYMBOLS))
    if policy_requirements['require_lowercase_characters']:
        password_composition.extend(choose_n_chars(char_num_groups.pop(0), LOWERCASE))
    if policy_requirements['require_uppercase_characters']:
        password_composition.extend(choose_n_chars(char_num_groups.pop(0), UPPERCASE))
    if policy_requirements['require_numbers']:
        password_composition.extend(choose_n_chars(char_num_groups.pop(0), NUMBERS))

    # Make sure our character types are mixed.
    shuffle(password_composition)
    return ''.join(password_composition)



from setuptools import find_packages, setup

setup(
    name='aws-auth-helper',
    long_description=open('README.rst').read(),
    description='Helper library providing ArgumentParser and Credentials class for AWS authentication',
    version='1.2.4',
    install_requires=['boto3'],
    packages=find_packages(),
    tests_requires=['unittest2','mock'],
    url='http://github.com/cloudgun/aws-auth-helper',
    license='GPLv2',
    author='Drew J. Sonne',
    author_email='drew.sonne@gmail.com',
)



from argparse import Namespace
from awsauthhelper.password import generate
from tests import AuthHelperTestCase

__author__ = 'drews'


class TestPassword(AuthHelperTestCase):
    """
    Make sure that our password generation has enough entropy.
    """
    SYMBOLS = list("!@#$%^&*()_+-=[]{}|'")
    LOWERCASE = list("abcdefghijklmnopqrstuvwxyz")
    UPPERCASE = list("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
    NUMBERS = list("0123456789")

    class MockPasswordPolicy(Namespace):
        def __init__(self, **kwargs):
            kwargs.setdefault('minimum_password_length', 6)
            kwargs.setdefault('require_symbols', False)
            kwargs.setdefault('require_lowercase_characters', False)
            kwargs.setdefault('require_uppercase_characters', False)
            kwargs.setdefault('require_numbers', False)

            super(TestPassword.MockPasswordPolicy, self).__init__(**kwargs)

    def test_generate_length(self):
        # Test default
        new_password = generate(self.MockPasswordPolicy())
        self.assertGreaterEqual(len(new_password), 6)
        self.assertIsInstance(new_password, basestring)

        # Test override
        new_password = generate(self.MockPasswordPolicy(
                minimum_password_length=10
        ))
        self.assertGreaterEqual(len(new_password), 10)
        self.assertIsInstance(new_password, basestring)

    def test_generate_symbols(self):
        new_password = generate(self.MockPasswordPolicy(
                require_symbols=True
        ))
        self.assertAnyIn(self.SYMBOLS, new_password)
        self.assertIsInstance(new_password, basestring)

    def test_generate_lowercase(self):
        new_password = generate(self.MockPasswordPolicy(
                require_lowercase_characters=True
        ))
        self.assertAnyIn(self.LOWERCASE, new_password)
        self.assertIsInstance(new_password, basestring)

    def test_generate_uppercase(self):
        new_password = generate(self.MockPasswordPolicy(
                require_uppercase_characters=True
        ))
        self.assertAnyIn(self.UPPERCASE, new_password)
        self.assertIsInstance(new_password, basestring)

    def test_generate_numbers(self):
        new_password = generate(self.MockPasswordPolicy(
                require_numbers=True
        ))
        self.assertAnyIn(self.NUMBERS, new_password)
        self.assertIsInstance(new_password, basestring)

    def test_generate_secure(self):
        new_password = generate(self.MockPasswordPolicy(
                minimum_password_length=12,
                require_symbols=True,
                require_lowercase_characters=True,
                require_uppercase_characters=True,
                require_numbers=True
        ))
        self.assertGreaterEqual(len(new_password), 12)
        self.assertAnyIn(self.SYMBOLS, new_password)
        self.assertAnyIn(self.LOWERCASE, new_password)
        self.assertAnyIn(self.UPPERCASE, new_password)
        self.assertAnyIn(self.NUMBERS, new_password)
        self.assertIsInstance(new_password, basestring)
